@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- $Id: battstat.h,v 1.37 2005/12/04 08:27:54 ryanl Exp $
+ $Id: battstat.h,v 1.39 2006/09/22 00:20:32 ryanl Exp $
  */
 
 #ifndef _battstat_h_
@@ -145,8 +145,9 @@ typedef struct _ProgressData {
   /* the current layout of the visual elements inside the table */
   LayoutConfiguration layout;
 
-  /* gtk_timeout identifier */
-  int pixtimer;
+  /* g_timeout source identifier */
+  int timeout_id;
+  int timeout;
 
   /* last_* for the benefit of the check_for_updates function */
   guint last_batt_life;
@@ -166,7 +167,7 @@ void battstat_show_help( ProgressData *battstat, const char *section );
 
 /* power-management.c */
 const char *power_management_getinfo( BatteryStatus *status );
-const char *power_management_initialise( int no_hal );
+const char *power_management_initialise (int no_hal, void (*callback) (void));
 void power_management_cleanup( void );
 int power_management_using_hal( void );
 
