@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- $Id: properties.c,v 1.51 2005/11/16 15:50:37 davyd Exp $
+ $Id: properties.c,v 1.51.6.1 2006/10/24 06:50:49 haip Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,9 +28,13 @@
 #include <sys/file.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
+#ifdef HAVE_SYS_SYSCTL_H
 #include <sys/sysctl.h>
+#endif
 
+#ifdef HAVE_ERR_H
 #include <err.h>
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <sys/file.h>
