@@ -68,6 +68,8 @@ struct _CPUFreqApplet {
         GdkPixbuf        *pixbufs[5];
         GtkTooltips      *tips;
 
+	gboolean          need_refresh;
+
         CPUFreqPrefs     *prefs;
         CPUFreqPopup     *popup;
 };
@@ -213,9 +215,8 @@ cpufreq_applet_init (CPUFreqApplet *applet)
         gtk_widget_show (applet->container);
            
         applet->tips = gtk_tooltips_new ();
-           
-        for (i = 0; i <= 4; i++)
-                applet->pixbufs[i] = NULL;
+
+	applet->need_refresh = TRUE;
 
         panel_applet_set_flags (PANEL_APPLET (applet), PANEL_APPLET_EXPAND_MINOR);
 
@@ -530,11 +531,6 @@ cpufreq_applet_about_cb (BonoboUIComponent *uic,
                 "Pablo Arroyo Loma <zzioma@yahoo.es>",
                 NULL
         };
-        GdkPixbuf          *pixbuf = NULL;
-
-        pixbuf = gdk_pixbuf_new_from_file_at_size (
-                GNOME_PIXMAPSDIR "/cpufreq-applet/cpufreq-applet.png",
-                48, 48, NULL);
 
         gtk_show_about_dialog (NULL,
                                "name",          _("CPU Frequency Scaling Monitor"),
@@ -546,11 +542,8 @@ cpufreq_applet_about_cb (BonoboUIComponent *uic,
                                "documenters",   documenters,
                                "artists",       artists, 
                                "translator-credits",    _("translator-credits"),
-                               "logo",          pixbuf,
+			       "logo-icon-name",        "gnome-cpu-frequency-applet",
                                NULL);
-
-        if (pixbuf)
-                g_object_unref (pixbuf);
 }
 
 static void
@@ -671,7 +664,6 @@ cpufreq_applet_update (CPUFreqApplet *applet, CPUFreqMonitor *monitor)
         gint            perc;
         guint           cpu;
         gchar          *governor;
-        static gboolean do_refresh = TRUE;
 
         cpu = cpufreq_monitor_get_cpu (monitor);
         freq = cpufreq_monitor_get_frequency (monitor);
@@ -721,9 +713,9 @@ cpufreq_applet_update (CPUFreqApplet *applet, CPUFreqMonitor *monitor)
         g_free (text_tip);
 
         /* Call refresh only the first time */
-        if (do_refresh) {
+        if (applet->need_refresh) {
                 cpufreq_applet_refresh (applet);
-                do_refresh = FALSE;
+		applet->need_refresh = FALSE;
         }
 }
 
@@ -795,6 +787,7 @@ cpufreq_applet_refresh (CPUFreqApplet *applet)
                         g_object_ref (applet->unit_label);
                         gtk_container_remove (GTK_CONTAINER (child), applet->unit_label);
                 }
+                g_list_free (children);
                 gtk_widget_destroy (applet->box);
         }
 
@@ -842,6 +835,15 @@ cpufreq_applet_refresh (CPUFreqApplet *applet)
 }
 
 /* Preferences callbacks */
+static void
+cpufreq_applet_prefs_cpu_changed (CPUFreqPrefs  *prefs,
+				  GParamSpec    *arg1,
+				  CPUFreqApplet *applet)
+{
+	cpufreq_monitor_set_cpu (applet->monitor,
+				 cpufreq_prefs_get_cpu (applet->prefs));
+}
+
 static void
 cpufreq_applet_prefs_show_mode_changed (CPUFreqPrefs  *prefs,
                                         GParamSpec    *arg1,
@@ -859,8 +861,8 @@ cpufreq_applet_setup (CPUFreqApplet *applet)
 
         glade_gnome_init ();
 
-        gtk_window_set_default_icon_from_file (GNOME_PIXMAPSDIR"/cpufreq-applet/cpufreq-applet.png", NULL);
-
+	gtk_window_set_default_icon_name ("gnome-cpu-frequency-applet");
+	
         panel_applet_add_preferences (PANEL_APPLET (applet),
                                       "/schemas/apps/cpufreq-applet/prefs", NULL);
            
@@ -872,6 +874,10 @@ cpufreq_applet_setup (CPUFreqApplet *applet)
         applet->prefs = cpufreq_prefs_new (prefs_key);
         g_free (prefs_key);
 
+	g_signal_connect (G_OBJECT (applet->prefs),
+			  "notify::cpu",
+			  G_CALLBACK (cpufreq_applet_prefs_cpu_changed),
+			  (gpointer) applet);
         g_signal_connect (G_OBJECT (applet->prefs),
                           "notify::show-mode",
                           G_CALLBACK (cpufreq_applet_prefs_show_mode_changed),
